@@ -44,6 +44,8 @@ impl FieldSpec {
 }
 #[derive(Clone)]
 struct StructSpec {
+    #[allow(dead_code)]
+    visibility: Visibility,
     attrs: Vec<Attribute>,
     recursive_attrs: Vec<Attribute>,
     ident: Ident,
@@ -85,7 +87,7 @@ impl StructSpec {
         list
     }
     fn structure(&self) -> impl quote::ToTokens {
-        unzip_n::unzip_n!(9);
+        unzip_n::unzip_n!(10);
         let ident = &self.ident;
         let mut notifying = false;
         let sattrs: Vec<_> = self
@@ -109,6 +111,7 @@ impl StructSpec {
             constructor_rec_validations,
             serde_match,
             get_match,
+            json_get_match,
             get_keys,
         ) = self
             .fields
@@ -144,13 +147,15 @@ impl StructSpec {
                     quote! {Self::#validate_id_rec(self.#id())},
                     serde_match(id, spec, &str_id, set_id, field_name),
                     get_match(spec, id, &str_id, field_name),
+                    json_get_match(spec, id, &str_id, field_name),
                     keys_match(spec, field_name, &str_id),
                 )
             })
             .collect::<Vec<_>>()
             .into_iter()
             .unzip_n_vec();
-        let serde_access = serde_access(ident, &serde_match, &get_match, &get_keys);
+        let serde_access =
+            serde_access(ident, &serde_match, &get_match, &get_keys, &json_get_match);
         let constructor_validations = constructor_validations
             .into_iter()
             .flatten()
@@ -217,7 +222,20 @@ fn serde_access(
     serde_match: &[proc_macro2::TokenStream],
     get_match: &[proc_macro2::TokenStream],
     get_keys: &[proc_macro2::TokenStream],
+    json_get_match: &[proc_macro2::TokenStream],
 ) -> Option<proc_macro2::TokenStream> {
+    let get_json = cfg!(feature = "serde_json").then(|| {
+        quote! {
+            fn get_json(& self, key: &str) -> Result<String, validated_struct::GetError>{
+                use std::any::Any;
+                match validated_struct::split_once(key, #SEPARATOR) {
+                    #(#json_get_match)*
+                    ("", key) if !key.is_empty() => self.get_json(key),
+                    _ => Err(validated_struct::GetError::NoMatchingKey),
+                }
+            }
+        }
+    });
     cfg!(feature = "serde").then(|| quote! {
         impl #ident {
             pub fn from_deserializer<'d, D: serde::Deserializer<'d>>(
@@ -238,6 +256,9 @@ fn serde_access(
                 }
             }
         }
+        impl<'a> validated_struct::ValidatedMapAssociatedTypes<'a> for #ident {
+            type Accessor = &'a dyn std::any::Any;
+        }
         impl validated_struct::ValidatedMap for #ident {
             fn insert<'d, D: serde::Deserializer<'d>>(&mut self, key: &str, value: D) -> Result<(), validated_struct::InsertionError>
             where
@@ -256,6 +277,7 @@ fn serde_access(
                     _ => Err(validated_struct::GetError::NoMatchingKey),
                 }
             }
+            #get_json
             type Keys = std::vec::Vec<String>;
             fn keys(&self) -> Self::Keys {
                 let mut keys = std::vec::Vec::new();
@@ -349,6 +371,23 @@ fn get_match(
     }
 }
 
+fn json_get_match(
+    spec: &FieldSpec,
+    id: &Ident,
+    str_id: &str,
+    field: &Ident,
+) -> proc_macro2::TokenStream {
+    let get_exact = quote! {(#str_id, "") => serde_json::to_string(self.#id()).map_err(|e| validated_struct::GetError::Other(e.into())),};
+    if spec.recursive_accessors() {
+        quote! {
+            #get_exact
+            (#str_id, key) => self.#field.get_json(key),
+        }
+    } else {
+        get_exact
+    }
+}
+
 fn serde_match(
     id: &Ident,
     spec: &FieldSpec,
